
from django.conf import settings
from django.contrib import admin
from django.contrib.admin.util import unquote
from django.core.exceptions import ValidationError
from django.db import models
from django.db.models.sql.constants import LOOKUP_SEP
from django.forms.formsets import BaseFormSet
from django.forms.models import (_get_foreign_key, model_to_dict, ModelForm,
    modelform_factory)
from django.forms.widgets import Select, SelectMultiple
from django.http import Http404, HttpResponseBadRequest, HttpResponse
from django.template.base import TemplateDoesNotExist
from django.template.defaultfilters import escape
from django.template.loader import get_template
from django.template.loader_tags import ExtendsNode
from django.utils import simplejson
from django.utils.encoding import force_unicode
from django.utils.functional import Promise, update_wrapper
from django.utils.safestring import SafeUnicode

from dynamic_choices.forms import DynamicModelForm, dynamic_model_form_factory
from dynamic_choices.forms.fields import DynamicModelChoiceField

try:
    static_url = settings.STATIC_URL
except AttributeError:
    try:
        static_url = settings.MEDIA_URL
    except AttributeError:
        static_url = ''

class LazyEncoder(simplejson.JSONEncoder):
    """
        Encoder used for 
    """
    def default(self, obj):
        if isinstance(obj, Promise):
            return force_unicode(obj)
        return super(LazyEncoder, self).default(obj)
lazy_encoder = LazyEncoder()


def get_dynamic_choices_from_form(form):
    fields = {}
    if form.prefix:
        prefix = "%s-%s" % (form.prefix, '%s')
    else:
        prefix = '%s'
    for name, field in form.fields.iteritems():
        if isinstance(field, DynamicModelChoiceField):
            widget_cls = field.widget.widget.__class__
            if widget_cls in (Select, SelectMultiple):
                widget = 'default'
            else:
                widget = "%s.%s" % (widget_cls.__module__,
                                    widget_cls.__name__)
            fields[prefix % name] = {
                                     'widget': widget,
                                     'value': list(field.widget.choices)
                                     }
    return fields

def dynamic_formset_factory(fieldset_cls, initial):
    class cls(fieldset_cls):
        def _construct_forms(self):
            "Append initial data for every single form"
            store = getattr(self, 'initial', None)
            if store is None:
                store = []
                setattr(self, 'initial', store)
            for i in xrange(self.total_form_count()):
                try:
                    actual = store[i]
                    actual.update(initial)
                except (ValueError, IndexError):
                    store.insert(i, initial)
            return super(cls, self)._construct_forms()
        
        def _get_empty_form(self, **kwargs):
            defaults = {'initial': initial}
            defaults.update(kwargs)
            return super(cls, self)._get_empty_form(**defaults)
        empty_form = property(_get_empty_form)
            
    cls.__name__ = "Dynamic%s" % fieldset_cls.__name__
    return cls

def dynamic_inline_factory(inline_cls):
    "Make sure the inline has a dynamic form"
    form_cls = getattr(inline_cls, 'form', None)
    if form_cls is ModelForm:
        form_cls = DynamicModelForm
    elif issubclass(form_cls, DynamicModelForm):
        return inline_cls
    else:
        form_cls = dynamic_model_form_factory(form_cls)
    
    class cls(inline_cls):
        form = form_cls
        
        def get_formset(self, request, obj=None, **kwargs):
            formset = super(cls, self).get_formset(request, obj=None, **kwargs)
            if not isinstance(formset.form(), DynamicModelForm):
                raise Exception('DynamicAdmin inlines\'s formset\'s form must be an instance of DynamicModelForm')
            return formset

    cls.__name__ = "Dynamic%s" % inline_cls.__name__
    return cls

def template_extends(template, expected_parent_name):
    try:
        tpl = get_template(template)
    except TemplateDoesNotExist:
        return False
    
    if (len(tpl.nodelist) and
        isinstance(tpl.nodelist[0], ExtendsNode)):
        node = tpl.nodelist[0]
        if node.parent_name == expected_parent_name:
            return True
        else:
            return template_extends(node.parent_name, expected_parent_name)
    else:
        return False

def dynamic_admin_factory(admin_cls):
    
    change_form_template = 'admin/dynamic_choices/change_form.html'
    
    class meta_cls(admin_cls.__metaclass__):
        "Metaclass that ensure form and inlines are dynamic"
        def __new__(cls, name, bases, attrs):
            # If there's already a form defined we make sure to subclass it
            if 'form' in attrs:
                attrs['form'] = dynamic_model_form_factory(attrs['form'])
            else:
                attrs['form'] = DynamicModelForm
            
            # Make sure the specified add|change_form_template
            # extends "admin/dynamic_choices/change_form.html"
            for t, default in {'add_form_template': None, 'change_form_template': change_form_template}.iteritems():
                if t in attrs:
                    if not template_extends(attrs[t], change_form_template):
                        raise Exception("Make sure specified %s.%s='%s' template extends '%s' "
                                        "in order to enabled DynamicAdmin" % (name, t, attrs[t],
                                                                              change_form_template))
                else:
                    attrs[t] = default
            
            # If there's some inlines defined we make sure that their form is dynamic
            # see dynamic_inline_factory
            if 'inlines' in attrs:
                attrs['inlines'] = [dynamic_inline_factory(inline_cls) for inline_cls in attrs['inlines']]
            
            return super(meta_cls, cls).__new__(cls, name, bases, attrs)
    
    class cls(admin_cls):
        
        __metaclass__ = meta_cls
    
        def _media(self):
            media = super(cls, self)._media()
            media.add_js(["%sjs/dynamic-choices.js" % static_url,
                          "%sjs/dynamic-choices-admin.js" % static_url])
            return media
        media = property(_media)
    
        def get_urls(self):
            # Inspired by
            # https://github.com/django-extensions/django-extensions/blob/master/django_extensions/admin/__init__.py
            from django.conf.urls.defaults import patterns, url
    
            def wrap(view):
                def wrapper(*args, **kwargs):
                    return self.admin_site.admin_view(view)(*args, **kwargs)
                return update_wrapper(wrapper, view)
    
            info = self.model._meta.app_label, self.model._meta.module_name
    
            urlpatterns = patterns('',
                url(r'(?:add|(?P<object_id>\w+))/choices/$',
                    wrap(self.dynamic_choices),
                    name="%s_%s_dynamic_admin" % info),
            ) + super(cls, self).get_urls()
    
            return urlpatterns

        def get_dynamic_choices_binder(self, request):
    
            id = lambda field: "[name='%s']" % field
            inline_field_selector = lambda fieldset, field: "[name^='%s-'][name$='-%s']" % (fieldset, field)
            
            fields = {}
            def add_fields(to_fields, to_field, bind_fields):
                if not (to_field in to_fields):
                    to_fields[to_field] = set()
                to_fields[to_field].update(bind_fields)
            
            app_name, model_name = self.model._meta.app_label, self.model._meta.module_name
            
            # Use get_form in order to allow formfield override
            # We should create a fake request from referer but all this
            # hack will be fixed when the code is embed directly in the page
            form = self.get_form(request)()
            rels = form.get_dynamic_relationships()
            for rel in rels:
                field_name = rel.split(LOOKUP_SEP)[0]
                if rel in form.fields:
                    add_fields(fields, id(field_name), [id(field) for field in rels[rel] if field in form.fields])
                    
            inlines = {}
            for formset in self.get_formsets(request):
                inline = {}
                formset_form = formset.form()
                inline_rels = formset_form.get_dynamic_relationships()
                prefix = formset.get_default_prefix()
                for rel in inline_rels:
                    if LOOKUP_SEP in rel:
                        base, field = rel.split(LOOKUP_SEP)[0:2]
                        if base == model_name and field in form.fields:
                            add_fields(fields, id(field), [inline_field_selector(prefix, field) \
                                                           for field in inline_rels[rel] if field in formset_form.fields])
                        elif base in formset_form.fields:
                            add_fields(inline, base, inline_rels[rel])
                    elif rel in formset_form.fields:
                        add_fields(inline, rel, inline_rels[rel])
                if len(inline):
                    inlines[prefix] = inline
                  
            # Replace sets in order to allow JSON serialization        
            for field, bindeds in fields.iteritems():
                fields[field] = list(bindeds)
                
            for fieldset, inline_fields in inlines.iteritems():
                for field, bindeds in inline_fields.iteritems():
                    inlines[fieldset][field] = list(bindeds)
                
            return SafeUnicode(u"django.dynamicAdmin(%s, %s);" % (simplejson.dumps(fields),
                                                                  simplejson.dumps(inlines)))
        
        def dynamic_choices(self, request, object_id=None):
                    
            opts = self.model._meta
            obj = self.get_object(request, object_id)
            # Make sure the specified object exists
            if object_id is not None and obj is None:
                raise Http404(_('%(name)s object with primary key %(key)r does not exist.') % {
                              'name': force_unicode(opts.verbose_name), 'key': escape(object_id)})
            
            form = self.get_form(request)(request.GET, instance=obj)
            data = get_dynamic_choices_from_form(form)
    
            for formset in self.get_formsets(request, obj):
                prefix = formset.get_default_prefix()
                try:
                    fs = formset(request.GET, instance=obj)
                    forms = fs.forms + [fs.empty_form]
                except ValidationError:
                    return HttpResponseBadRequest("Missing %s ManagementForm data" % prefix)
                for form in forms:
                    data.update(get_dynamic_choices_from_form(form))
                    
            if 'DYNAMIC_CHOICES_FIELDS' in request.GET:
                fields = request.GET.get('DYNAMIC_CHOICES_FIELDS').split(',')
                for field in data.keys():
                    if not (field in fields):
                        del data[field]
            
            return HttpResponse(lazy_encoder.encode(data), mimetype='application/json')
        
        # Make sure to pass request data to fieldsets
        # so they can use it to define choices
        def get_formsets(self, request, obj=None):
            initial = {}
            model = self.model
            opts = model._meta
            data = getattr(request, request.method).items()
            # If an object is provided we collect data
            if obj is not None:
                initial.update(model_to_dict(obj))
            # Make sure to collect parent model data
            # and provide it to fieldsets in the form of
            # parent__field from request if its provided.
            # This data should be more "up-to-date".
            for k, v in data:
                if v:
                    try:
                        f = opts.get_field(k)
                    except models.FieldDoesNotExist:
                        continue
                    if isinstance(f, models.ManyToManyField):
                        initial[k] = v.split(",")
                    else:
                        initial[k] = v
                        
            try:
                # Django >= 1.4
                inline_instances = self.get_inline_instances(request)
            except AttributeError:
                # Django < 1.4
                inline_instances = self.inline_instances
            
            for formset, inline in zip(super(cls, self).get_formsets(request, obj), inline_instances):
                fk = _get_foreign_key(self.model, inline.model, fk_name=inline.fk_name).name
                fk_initial = dict(('%s__%s' % (fk, k),v) for k, v in initial.iteritems())
                # If we must provide additional data
                # we must wrap the formset in a subclass
                # because passing 'initial' key argument is intercepted
                # and not provided to subclasses by BaseInlineFormSet.__init__
                if len(initial):
                    formset = dynamic_formset_factory(formset, fk_initial)
                yield formset
                
        def add_view(self, request, form_url='', extra_context=None):
            context = {'dynamic_choices_binder': self.get_dynamic_choices_binder(request)}
            context.update(extra_context or {})
            return super(cls, self).add_view(request, form_url='', extra_context=context)

        def change_view(self, request, object_id, extra_context=None):
            context = {'dynamic_choices_binder': self.get_dynamic_choices_binder(request)}
            context.update(extra_context or {})
            return super(cls, self).change_view(request, object_id, extra_context=context)

    return cls

DynamicAdmin = dynamic_admin_factory(admin.ModelAdmin)

try:
    from reversion.admin import VersionAdmin
    DynamicVersionAdmin = dynamic_admin_factory(VersionAdmin)
except ImportError:
    pass



import inspect

from django.core import exceptions
from django.core.exceptions import FieldError
from django.db.models import ForeignKey, ManyToManyField, OneToOneField
from django.db.models.base import Model
from django.db.models.fields import FieldDoesNotExist, Field
from django.db.models.fields.related import add_lazy_relation
from django.db.models.sql.constants import LOOKUP_SEP
from django.db.models.signals import class_prepared
from django.forms.models import model_to_dict

from dynamic_choices.db.query import dynamic_queryset_factory, unionize_querysets
from dynamic_choices.forms.fields import (DynamicModelChoiceField,
    DynamicModelMultipleChoiceField)

class DynamicChoicesField(object):
    
    def __init__(self, *args, **kwargs):
        super(DynamicChoicesField, self).__init__(*args, **kwargs)
        # Hack to bypass non iterable choices validation
        if isinstance(self._choices, basestring) or callable(self._choices):
            self._choices_callback = self._choices
            self._choices = []
        else:
            self._choices_callback = None
        self._choices_relationships = None
    
    def contribute_to_class(self, cls, name):
        super(DynamicChoicesField, self).contribute_to_class(cls, name)
        
        if self._choices_callback is not None:
            class_prepared.connect(self.__validate_definition,
                                   sender=cls)
    
    def __validate_definition(self, *args, **kwargs):
        def error(message):
            raise FieldError("%s: %s: %s" % (self.related.model._meta, self.name, message))

        original_choices_callback = self._choices_callback

        # The choices we're defined by a string
        # therefore it should be a cls method
        if isinstance(self._choices_callback, basestring):
            callback = getattr(self.related.model, self._choices_callback, None)
            if not callable(callback):
                error('Cannot find method specified by choices.')
            args_length = 2 # Since the callback is a method we must emulate the 'self'
            self._choices_callback = callback
        else:
            args_length = 1 # It's a callable, it needs no reference to model instance
        
        spec = inspect.getargspec(self._choices_callback)
        
        # Make sure the callback has the correct number or arg
        if spec.defaults is not None:
            spec_defaults_len = len(spec.defaults)
            args_length += spec_defaults_len
            self._choices_relationships = spec.args[-spec_defaults_len:]
        else:
            self._choices_relationships = []
        
        if len(spec.args) != args_length:
            error('Specified choices callback must accept only a single arg')
        
        self._choices_callback_field_descriptors = {}
        
        # We make sure field descriptors are valid
        for descriptor in self._choices_relationships:
            lookups = descriptor.split(LOOKUP_SEP)
            meta = self.related.model._meta
            depth = len(lookups)
            step = 1
            fields = []
            for lookup in lookups:
                try:
                    field = meta.get_field(lookup)
                    # The field is a foreign key to another model
                    if isinstance(field, ForeignKey):
                        try:
                            meta = field.rel.to._meta
                        except AttributeError:
                            # The model hasn't been loaded yet
                            # so we must stop here and start over
                            # when it is loaded.
                            if isinstance(field.rel.to, basestring):
                                self._choices_callback = original_choices_callback
                                return add_lazy_relation(field.model, field,
                                                         field.rel.to,
                                                         self.__validate_definition)
                            else:
                                raise
                        step += 1
                    # We cannot go deeper if it's not a model
                    elif step != depth:
                        error('Invalid descriptor "%s", "%s" is not a ForeignKey to a model' % (
                               LOOKUP_SEP.join(lookups), LOOKUP_SEP.join(lookups[:step])))
                    fields.append(field)
                except FieldDoesNotExist:
                    # Lookup failed, suggest alternatives
                    depth_descriptor = LOOKUP_SEP.join(descriptor[:step - 1])
                    if depth_descriptor:
                        depth_descriptor += LOOKUP_SEP
                    choice_descriptors = [(depth_descriptor + name) for name in meta.get_all_field_names()]
                    error('Invalid descriptor "%s", choices are %s' % (
                          LOOKUP_SEP.join(descriptor), ', '.join(choice_descriptors)))
            
            self._choices_callback_field_descriptors[descriptor] = fields
    
    @property
    def has_choices_callback(self):
        return callable(self._choices_callback)
    
    @property
    def choices_relationships(self):
        return self._choices_relationships
    
    def _invoke_choices_callback(self, model_instance, qs, data):
        args = [qs]
        # Make sure we pass the instance if the callback is a class method
        if inspect.ismethod(self._choices_callback):
            args.insert(0, model_instance)
        
        values = {}
        for descriptor, fields in self._choices_callback_field_descriptors.items():
            depth = len(fields)
            step = 1
            lookup_data = data
            value = None
            
            # Direct lookup
            # foo__bar in data
            if descriptor in data:
                value = data[descriptor]
                step = depth
                field = fields[-1]
            else:
                # We're going to try to lookup every step of the descriptor.
                # We first try field1, then field1__field2, etc..
                # When there's a match we start over with fieldmatch and set the lookup data
                # to the matched value.
                field_name = "%s"
                for field in fields:
                    field_name = field_name % field.name
                    if field_name in lookup_data:
                        value = lookup_data[field_name]
                        if step != depth:
                            if isinstance(field, ManyToManyField):
                                # We cannot lookup in m2m, it must be the final step
                                break
                            elif isinstance(value, list):
                                value = value[0] # Make sure we've got a scalar
                            if isinstance(field, ForeignKey):
                                if not isinstance(value, Model):
                                    try:
                                        value = field.rel.to.objects.get(pk=value)
                                    except Exception:
                                        # Invalid object
                                        break
                                lookup_data = model_to_dict(value)
                                field_name = "%s"
                            step += 1
                    elif step != depth:
                        field_name = "%s%s%s" % (field_name, LOOKUP_SEP, '%s')
            
            # We reached descriptors depth
            if step == depth:
                if isinstance(value, list) and \
                    not isinstance(field, ManyToManyField):
                    value = value[0] # Make sure we've got a scalar if its not a m2m
                # Attempt to cast value, if failed we don't assign since it's invalid
                try:
                    values[descriptor] = field.to_python(value)
                except:
                    pass
        
        return self._choices_callback(*args, **values)

    def __super(self):
        # Dirty hack to allow both DynamicChoicesForeignKey and DynamicChoicesManyToManyField
        # to inherit this behavior with multiple inheritance
        for base in self.__class__.__bases__:
            if issubclass(base, Field):
                self.__super = lambda : base #cache
                return base
        raise Exception('Subclasses must inherit from atleast one subclass of django.db.fields.Field')

    def formfield(self, **kwargs):
        if self.has_choices_callback:
            db = kwargs.pop('using', None)
            qs = self.rel.to._default_manager.using(db).complex_filter(self.rel.limit_choices_to)
            defaults = {
                'using': db,
                'form_class': self.form_class,
                'queryset': dynamic_queryset_factory(qs, self)
            }
            defaults.update(kwargs)
        else:
            defaults = kwargs
            
        return self.__super().formfield(self, **defaults)

class DynamicChoicesForeignKeyMixin(DynamicChoicesField):
    
    def validate(self, value, model_instance):
        if self.has_choices_callback:
            if self.rel.parent_link:
                return
            if value is None:
                return
            
            data = model_to_dict(model_instance)
            for field in model_instance._meta.fields:
                try:
                    data[field.name] = getattr(model_instance, field.name)
                except field.rel.to.DoesNotExist:
                    pass
            if model_instance.id:
                for m2m in model_instance._meta.many_to_many:
                    data[m2m.name] = getattr(model_instance, m2m.name).all()
            
            qs = self.rel.to._default_manager.filter(**{self.rel.field_name:value})
            qs = qs.complex_filter(self.rel.limit_choices_to)
            
            dcqs = self._invoke_choices_callback(model_instance, qs, data)
            # If a tuple is provided we must build
            # a new Queryset by combining group's ones
            if isinstance(dcqs, tuple):
                qs = qs and unionize_querysets(q[1] for q in dcqs)
            else:
                qs = dcqs
            
            if not qs.exists():
                raise exceptions.ValidationError(self.error_messages['invalid'] % {
                    'model': self.rel.to._meta.verbose_name, 'pk': value})
        else:
            super(DynamicChoicesForeignKey, self).validate(value, model_instance)

class DynamicChoicesForeignKey(DynamicChoicesForeignKeyMixin, ForeignKey):

    form_class = DynamicModelChoiceField

class DynamicChoicesOneToOneField(DynamicChoicesForeignKeyMixin, OneToOneField):
    
    form_class = DynamicModelChoiceField

class DynamicChoicesManyToManyField(DynamicChoicesField, ManyToManyField):
    
    form_class = DynamicModelMultipleChoiceField
    
try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ['^dynamic_choices\.db\.models'])
except ImportError:
    pass
    


import operator

from django.db.models.query import EmptyQuerySet, QuerySet


class EmptyDynamicChoicesQueryset(EmptyQuerySet):
    
    def filter_for_instance(self):
        return self

class DynamicChoicesQueryset(QuerySet):
    
    def _clone(self, *args, **kwargs):
        clone = super(DynamicChoicesQueryset, self)._clone(*args, **kwargs)
        clone._field = self._field
        return clone
        
    def filter_for_instance(self, instance, data):
        return self._field._invoke_choices_callback(instance, self, data)
    
    def none(self):
        return self._clone(klass=EmptyDynamicChoicesQueryset)

def dynamic_queryset_factory(queryset, field):
    
    clone = queryset._clone(DynamicChoicesQueryset)
    clone._field = field
    
    return clone

def unionize_querysets(querysets):
    """
    Combine querysets in a way that results
    from every queryset in joined together
    """
    return reduce(operator.or_, querysets)




from django.forms.models import ModelForm
from django.db.models.fields.related import ManyToManyField
from django.db.models.sql.constants import LOOKUP_SEP

from dynamic_choices.forms.fields import (DynamicModelChoiceField,
    DynamicModelMultipleChoiceField)

__all__ = ('DynamicModelForm', 'dynamic_model_form_factory')

def original_dynamic_model_form_factory(model_form_cls):
    class cls(model_form_cls):
        def __init__(self, *args, **kwargs):
            super(cls, self).__init__(*args, **kwargs)
            
            # Fetch initial data for initial
            data = self.initial.copy()
            
            # Update data if it's avaible
            for field in self.fields.iterkeys():
                raw_value = self._raw_value(field)
                if raw_value is not None:
                    if raw_value:
                        data[field] = raw_value
                    elif field in data:
                        del data[field]
                        
            # Bind instances to dynamic fields
            for field in self.fields.itervalues():
                if isinstance(field, DynamicModelChoiceField):
                    field.set_choice_data(self.instance, data)
                    
        def get_dynamic_relationships(self):
            rels = {}
            opts = self.instance._meta
            for name, field in self.fields.iteritems():
                #TODO: check for excludes?
                if isinstance(field, DynamicModelChoiceField):
                    for choice in opts.get_field(name).choices_relationships:
                        if not (choice in rels):
                            rels[choice] = set()
                        rels[choice].add(name)
            return rels
                    
    cls.__name__ = "Dynamic%s" % model_form_cls.__name__
    return cls

DynamicModelForm = original_dynamic_model_form_factory(ModelForm)

def dynamic_model_form_factory(model_form_cls):
    cls = original_dynamic_model_form_factory(model_form_cls)
    cls.__bases__ += (DynamicModelForm,)
    return cls
                



from django.forms.fields import ChoiceField
from django.forms.models import (ModelChoiceField, ModelChoiceIterator,
    ModelMultipleChoiceField)
    
from dynamic_choices.db.query import DynamicChoicesQueryset, unionize_querysets

class GroupedModelChoiceIterator(ModelChoiceIterator):
    def __init__(self, field):
        super(GroupedModelChoiceIterator, self).__init__(field)
        self.groups = field._groups
    
    def __iter__(self):
        if self.field.empty_label is not None:
            yield (u"", self.field.empty_label)
            
        #TODO: consider field.cache_choices
        
        for group in self.groups:
            label, qs = group
            yield (label, [self.choice(obj) for obj in qs])

    def __len__(self):
        return sum(len(group[1]) for group in self.groups)

class DynamicModelChoiceField(ModelChoiceField):
    
    def __init__(self, *args, **kwargs):
        self._instance = None
        self._data = {}
        self._groups = None
        super(DynamicModelChoiceField, self).__init__(*args, **kwargs)

    def _get_queryset(self):
        return self._queryset.distinct()

    def _set_queryset(self, queryset):
        self._original_queryset = queryset
        self._groups = None
        if self._instance and isinstance(queryset, DynamicChoicesQueryset):
            queryset = queryset.filter_for_instance(self._instance, self._data)
            if isinstance(queryset, tuple):
                self._groups = queryset
                queryset = unionize_querysets(q[1] for q in queryset)
        self._queryset = queryset
        self.widget.choices = self.choices

    queryset = property(_get_queryset, _set_queryset)
    
    def set_choice_data(self, instance, data):
        self._instance = instance
        self._data = data
        self.queryset = self._original_queryset
        
    def _get_choices(self):
        if isinstance(self._groups, tuple):
            return GroupedModelChoiceIterator(self)
        else:
            return super(DynamicModelChoiceField, self)._get_choices()

    choices = property(_get_choices, ChoiceField._set_choices)

class DynamicModelMultipleChoiceField(DynamicModelChoiceField, ModelMultipleChoiceField):
    pass



#!/usr/bin/python

from setuptools import setup, find_packages

from dynamic_choices import VERSION

github_url = 'https://github.com/charettes/django-dynamic-choices'
long_desc = '''
%s

%s
''' % (open('README').read(), open('CHANGELOG').read())

setup(
    name='django-dynamic-choices',
    version='.'.join(str(v) for v in VERSION),
    description='Django admin fk and m2m dynamic choices by providing callback support',
    long_description=long_desc,
    url=github_url,
    author='Simon Charette',
    author_email='charette.s@gmail.com',
    requires=[
        'django (>=1.3.1)'
    ],
    packages=find_packages(exclude=['testapp']),
    include_package_data=True,
    license='MIT License',
    classifiers=[
        'Development Status :: 1 - Planning',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
)
